#!/usr/bin/python3

# import json
from ast import In
from datetime import datetime, timedelta
import logging.config

from algo.data.common import CompletePriceStream, Symbol
# from algo.config import config
# from algo.constant import Symbol
# from algo.data.reader import PriceStream, get_data_reader
# from algo.simulation import simulate_trading
# from algo.strategy.moving_averages import MAStrategyParams, run_simulation
# from algo.trading import PricePoint
# from algo.util import d, now
# from algo.strategy import moving_average_sample
from algo.data.live import get_live_price_stream, get_complete_prices_generator
from algo.data.reader import get_data_reader
from algo.util import d
from algo.config import config
from algo.data.reporter import get_cloud_watch_reporter
from algo.scripts import cache_from_influx

logging.config.fileConfig(
    './logging.conf',
    disable_existing_loggers=False
)

log = logging.getLogger(__name__)


# def serialize_data_record(data_record):
#     """
#     input is a FluxRecord
#     """
#     return data_record['result']['_measurement']


# def read_all(data: PriceStream):
#     data_points_counter = 0
#     total_length = 0  # in chars
#     total_ask = 0.0
#     total_bid = 0.0
#     for data_point in data:
#         # log.info('data-point', extra=d(data_point))
#         # log.info('data-point', extra=d(data_point[0].isoformat()))
#         total_ask += data_point[1]
#         total_bid += data_point[2]

#         data_serialized = data_point.to_json()
#         total_length += len(data_serialized)
#         data_points_counter += 1
#     return data_points_counter, total_length, total_ask, total_bid


def try_listener():
    """write the fucking tests"""
    config.set_config({
    })
    price_stream: CompletePriceStream = get_live_price_stream(
        price_reader=get_complete_prices_generator)
    # influx_reporter = get_influx_data_reporter()
    cw_reporter = get_cloud_watch_reporter()
    for price in price_stream:
        log.debug('price-point', extra=d(p=price))
        # influx_reporter.report(price)
        cw_reporter.report(price)


def verify_cache_works():
    data_reader = get_data_reader()
    log.info('loading cached data')
    data_iterator = data_reader.day_data(
        '2022-09-01 00:00:00+00:00', Symbol.JPY
    )
    all_data = list(data_iterator)
    log.info('done')
    log.info('loading influx no cache data')
    no_cahe_iterator = data_reader.day_data_no_cache(
        '2022-09-01 00:00:00+00:00', Symbol.JPY
    )
    data_no_cache = list(no_cahe_iterator)
    log.info('done')
    assert all_data == data_no_cache
    # data_iterator = data_reader.week_data(
    #     '2022-09-04 00:00:00+00:00', Symbol.JPY
    # )

    # data_counter = 0
    # for d in data_iterator:
    #     data_counter += 1
    #     if data_counter % 100 == 0:
    #         log.info('data-counter', extra={'counter': data_counter})


def cache_data():
    data_reader = get_data_reader()
    log.info('loading cached data')
    for s in Symbol:
        log.info('loading data', extra={'s': s})
        data_iterator = data_reader.week_data(
            '2022-10-20 00:00:00+00:00', s
        )
        all_data = list(data_iterator)
        log.info(
            'got-data',
            extra={
                'len': len(all_data),
                'symbol': s
            }
        )


def try_dynamo_cache():
    from algo.data.cache.dynamo import DynamoCache  # pylint: disable=all
    dc = DynamoCache()
    dc.set('test-data', {'d': 12})
    assert {'d': 12} == dc.get('test-data')
    dc.set_lock('sv')
    assert dc.has_lock('sv')
    assert not dc.has_lock('asdf')
    dc.clear_lock('sv')
    assert not dc.has_lock('sv')


def cache_data_from_local_server():
    dr = get_data_reader()
    current_week = datetime.fromisoformat('2022-09-22 00:00:00+00:00')
    end_week = datetime.fromisoformat('2022-10-20 00:00:00+00:00')
    while current_week < end_week:
        log.info('requesting data for the week', extra={
            'week-of': current_week
        })
        for s in Symbol:
            log.info('loading data', extra={'s': s})
            data_iterator = dr.week_data(
                current_week.isoformat(), s
            )
            all_data = list(data_iterator)
            log.info(
                'got-data',
                extra={
                    'len': len(all_data),
                    'symbol': s,
                    'week': current_week
                }
            )

        current_week += timedelta(days=7)


if __name__ == '__main__':
    log.info('playground home will be here')
    cache_from_influx()
    # try_dynamo_cache()
    # try_listener()
    # cache_data()
    # cache_data_from_local_server()

    # price_generator = (
    #     PricePoint(now(), 1, 2),
    #     PricePoint(now(), 2, 3),
    #     PricePoint(now(), 3, 4),
    #     PricePoint(now(), 5, 6),
    # )
    # run_simulation(price_generator, MAStrategyParams(1, 2))
    # data_reader = get_data_reader()

    # data_iterator = data_reader.day_data(
    #     '2022-06-20 00:00:00+00:00', Symbol.JPY)
    # log.info('start-reading')
    # data_stat = read_all(data_iterator)
    # log.info('data-stat', extra=d(data_stat))
    # log.info('first-read-complete')
    # data_iterator = data_reader.day_data(
    #     '2022-06-20 00:00:00+00:00', Symbol.JPY)
    # log.info('start-second-reading')
    # data_stat = read_all(data_iterator)
    # log.info('data-stat', extra=d(data_stat))
    # log.info('second-read-complete')

    # strategy = moving_average_sample
    # result = simulate_trading(
    #     '2022-06-20',
    #     Symbol.JPY,
    #     strategy.module,
    #     strategy.args(2, 3)
    # )
    # log.info('simulation-complete', extra={'result': result})
